#include <avr/io.h>
#include <stdint.h>
#include "spi.h"
#include "registerstrings.h"
#include "other.h"
#define TIMEOUT_WAIT	0xFFUL		// So hoch wird gezhlt, bis Timeout erreicht wird


/*******************************************************
	Funktion :		Error-Funktion
	Eigenschaften:	Ruft allgemeine Error Routine auf
	bergabewert:	1 Byte Error Code
	Rckgabewert:	-
********************************************************/
void spi_error(uint8_t error_code)
{
	// ERROR-Routine
	programm_error(error_code);
}



/*******************************************************
	Funktion :		Whle Messchip aus
	Eigenschaften:	PB4 als Ausgang => auf low
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void spi_select_measuring_chip(void)
{
	PORTB |= (1 << PB3); 				// SD-Karte CS ausschalten
	PORTB &= ~(1 << PB4);				// Messchip CS einschalten (lschen von PB4)
}


/*******************************************************
	Funktion :		Deselektiere Messchip und SD-Karte
	Eigenschaften:	-
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void spi_unselect(void)
{
	PORTB |= (1 << PB3); 				// SD-Karte CS ausschalten
	PORTB |= (1 << PB4); 				// Messchip CS ausschalten (setzen von PB4)
}


/*******************************************************
	Funktion :		Whle SD-Karte aus
	Eigenschaften:	PB 3 als Ausgang => auf low
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void spi_select_sdcard(void)
{
	PORTB |= (1 << PB4); 				// Messchip CS ausschalten
	PORTB &= ~(1 << PB3);				// SD-Karte CS einschalten (lschen von PB3)
}




/*******************************************************
	Funktion :		Initialisiere SPI
	Eigenschaften:	SPI als Master, fck/16
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void spi_init(void)
{
	// Mosi + Clock als Output setzen
	DDR_SPI = (1 <<SPI_MOSI) | (1 <<SPI_CLK);
	DDR_SPI |= (1 << PB3) | ( 1 << PB4);	// 3 & 4 als Ausgang (SS)

	// Schalte SPI ein und fck/128
	SPSR &= ~(1 << SPI2X);
	SPCR = (1<<SPE) | (1<<MSTR) | (1<<SPR1) | (1<<SPR0) ;
}


/*******************************************************
	Funktion :		Sende ber SPI
	Eigenschaften:	Sendet 1 Byte Daten
	bergabewert:	1 Byte Daten
	Rckgabewert:	-
********************************************************/
void spi_send(uint8_t data) 
{	
	SPDR = data;						// Gib Daten in bertragungspuffer

	timeout_counter = 0;					// Setze sicherheitshalber zurck
	while(timeout_counter != TIMEOUT_WAIT)	// Timeout Schleife
	{
		if(SPSR & (1 << SPIF))				// bertragungspuffer leer?
		{	
			timeout_counter = 0;			// Setze wieder zurck (fr nchste berprfung
			return;
		}
		else
		{
			timeout_counter++;
		}
	}
	timeout_counter = 0;					// Setze wieder zurck (fr nchste berprfung
	spi_error(EC_SPI_SEND_TIMEOUT);			// Rufe Error-Funktion mit Errorcode auf
}



/*******************************************************
	Funktion :		Empfngt ber SPI
	Eigenschaften:	Empfange 1 Byte Daten
	bergabewert:	-
	Rckgabewert:	1 Byte Daten
********************************************************/
uint8_t spi_get(void) 
{	
	SPDR = 0xFF; 					// Lege in bertragungspuffer hinein

	// Warte bis fertig Empfangen
	timeout_counter = 0;					// Setze sicherheitshalber zurck
	while(timeout_counter != TIMEOUT_WAIT)	// Timeout Schleife
	{
		if(SPSR & (1 << SPIF))				// bertragungspuffer leer?
		{	
			timeout_counter = 0;			// Setze wieder zurck (fr nchste berprfung
			return SPDR;					// Return die Daten
		}
		else
		{
			timeout_counter++;
		}
	}
	timeout_counter = 0;					// Setze wieder zurck (fr nchste berprfung
	spi_error(EC_SPI_SEND_TIMEOUT);			// Rufe Error-Funktion mit Errorcode auf
	return -1;								// Return Error
}




/*******************************************************
	Funktion :		Empfngt & Sendet ber SPI
	Eigenschaften:	Empfange 1 Byte Daten und sendet 1 Byte
	bergabewert:	1 Byte Daten
	Rckgabewert:	1 Byte Daten
********************************************************/
uint8_t spi_transmit(uint8_t data) 
{	
	SPDR = data; 							// Lege in bertragungspuffer hinein

	// Warte bis fertig Empfangen
	timeout_counter = 0;					// Setze sicherheitshalber zurck
	while(timeout_counter != TIMEOUT_WAIT)	// Timeout Schleife
	{
		if(SPSR & (1 << SPIF))				// bertragungspuffer leer?
		{	
			timeout_counter = 0;			// Setze wieder zurck (fr nchste berprfung
			return SPDR;					// Return die Daten
		}
		else
		{
			timeout_counter++;
		}
	}
	timeout_counter = 0;					// Setze wieder zurck (fr nchste berprfung
	spi_error(EC_SPI_SEND_TIMEOUT);			// Rufe Error-Funktion mit Errorcode auf
	return -1;								// Return Error
}



/*******************************************************
	Funktion :		Synchronisation des CS5463 Messchip
	Eigenschaften:	Diese Funktion wird bentigt wenn die SCLK des Messchips aus dem Takt gert
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void spi_measuring_chip_synchronize(void) 
{
	spi_send_sync1();
	spi_send_sync1();
	spi_send_sync1();
	spi_send_sync0();
}

/*******************************************************
	Funktion :	Pageselect des CS5463 Messchip	
	Eigenschaften:	-
	bergabewert:	die gewnschte Seite auf die gewechselt werden soll
	Rckgabewert:	-
********************************************************/
void spi_measuring_chip_select_page(uint8_t page) 
{	
	spi_send(0x7E);			// Inititisiere write-Mode und whle Register 31 an
	spi_send(0x00);
	spi_send(0x00);
	spi_send(page);			// setze die gewnschte Page...
}



/*******************************************************
	Funktion :		Sende ein Start an Messchip
	Eigenschaften:	-
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void spi_send_start_conversions(void)
{
	spi_send(0b11101000);				// Sende Start mit lngerem Datenaustausch
}



/*******************************************************
	Funktion :		Sende SYNC0
	Eigenschaften:	-
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void spi_send_sync0(void)
{
	spi_send(0b11111110);				// Schicke ein SYNC0 Packet
}



/*******************************************************
	Funktion :		Sende SYNC1
	Eigenschaften:	-
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void spi_send_sync1(void)
{
	spi_send(0b11111111);				// Schicke ein SYNC1 Packet
}



/*******************************************************
	Funktion :		Sende Power-up/Halt
	Eigenschaften:	-
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void spi_send_power_up(void)
{
	spi_send(0b10100000);				// Schicke ein Power-up/Halt Packet
}



/*******************************************************
	Funktion :		Sende Software Reset
	Eigenschaften:	-
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void spi_send_software_reset(void)
{
	spi_send(0b10000000);				// Schicke ein Software-Reset Packet
}



/*******************************************************
	Funktion :		Schreibe Daten in Register
	Eigenschaften:	3 Byte Daten in Registeraddresse
	bergabewert:	1 Byte Addresse, 3 Byte Daten
	Rckgabewert:	-
********************************************************/
void spi_write_register(uint8_t register_address, uint32_t data)
{
	uint8_t temp = 0x00;
	temp = (1 << 6);							// Stelle Write-Bit ein
	register_address &= 0b00011111;				// Maskiere sicherheitshalber Register_addresse
	register_address = register_address << 1;	// Verschiebe um 1 nach links
	temp |= register_address;					// Fge beide Daten zusammen
	spi_send(temp);								// Sende fertiges Packet
	temp = data & 0xFF0000UL;					// Maskiere das hohe Byte Daten
	spi_send(temp);								// Sende das hohe Byte Daten
	temp = data & 0xFF00UL;						// Maskiere das mittlere Byte Daten
	spi_send(temp);								// Sende das mittlere Byte Daten
	temp = data & 0xFFUL;						// Maskiere das niedrige Byte Daten
	spi_send(temp);								// Sende das niedrige Byte Daten
}



/*******************************************************
	Funktion :		Lese Daten aus Register
	Eigenschaften:	3 Byte Daten aus Registeraddresse holen
	bergabewert:	1 Byte Addresse, 3 Pointer als Rckgabe
	Rckgabewert:	-
********************************************************/
uint32_t spi_read_register(uint8_t register_address)
{
	uint8_t temp = 0x00;
	uint32_t data = 0x00;

	register_address &= 0b00011111;				// Maskiere sicherheitshalber Register_addresse
	register_address = register_address << 1;	// Verschiebe um 1 nach links
	spi_send(register_address);					// Sende fertiges Packet
	data = spi_transmit(0xFF);					// Hole das hohe Byte Daten
	data = (data << 8);							// Schiebe hohes Byte in Mitte

	temp = spi_transmit(0xFF);					// Hole das Mittlere Byte
	data = data | temp;							// Fge mittleres+hohes Byte zusammen
	data = (data << 8);							// Schiebe hohes & Mittleres Byte auf richtigen Platz

	temp = spi_transmit(0xFF);					// Hole das niedrige Byte;
	data = data | temp;							// Fge niedrigstes Byte hinzu

	return data;
}


/*******************************************************
	Funktion :		Hole Messwerte von Messchip
	Eigenschaften:	holt nur Messwerte nach Messwertemaske
	bergabewert:	
	Rckgabewert:	-
********************************************************/
void spi_get_measured(uint8_t *measured_length, uint32_t measurement_mask_low, uint32_t measurement_mask_high, uint32_t *measured)
{
	*measured_length = 0;
	
	// Werte von Register Page 0 holen
	spi_measuring_chip_select_page(0);											// Stelle Register Page 0 ein
	for(uint8_t elementcounter = 0; elementcounter < 32; ++elementcounter)
	{
		if(measurement_mask_low & (0x01UL << elementcounter))
		{
			measured[*measured_length] = spi_read_register(elementcounter);		// Hole Messdaten
			//used_register_names[*measured_length] = elementcounter;				// Welches Register wurde gespeichert? (Fr Namen)
			++*(measured_length);
		}
	}

	// Werte von Register Page 1 holen
	spi_measuring_chip_select_page(1);											// Stelle Register Page 1 ein
	for(uint8_t elementcounter = 32; elementcounter < 36; ++elementcounter)
	{
		if(measurement_mask_high & (0x01UL << (elementcounter-32))) 			// -32 weil bei i=32 es Bit 0 in High Maske ist
		{
			measured[*measured_length] = spi_read_register(elementcounter-32);	// -32 weil bei i=32 es Addresse 0 hat
			//used_register_names[*measured_length] = elementcounter;				// Welches Register wurde gespeichert? (Fr Namen)
			++*(measured_length);
		}
	}

	// Werte von Register Page 3 holen
	spi_measuring_chip_select_page(3);											// Stelle Register Page 3 ein
	for(uint8_t elementcounter = 36; elementcounter < 40; ++elementcounter)
	{
		if(measurement_mask_high & (0x01UL << (elementcounter-32)))			// -32 weil bei i=36 es Bit 4(start bei 0) in High Mask ist
		{
			if(elementcounter <= 37)
			{
				measured[*measured_length] = spi_read_register(elementcounter-30);	// -30 weil bei i=36 es die Addresse 6 hat
			}
			else
			{
				measured[*measured_length] = spi_read_register(elementcounter-28);	// -28 weil bei i=38 es die Addresse 10 hat
			}
			//used_register_names[*measured_length] = elementcounter;				// Welches Register wurde gespeichert? (Fr Namen)
			++*(measured_length);
		}
	}
}
