#include <avr/io.h>
#include <stdint.h>
#include "general.h"
#include "usart.h"
#include "other.h"
#include <avr/interrupt.h>
#include <util/delay.h>

struct time pc,start,end;
uint8_t seq_nr;
uint8_t pc_seq_nr;
uint8_t checksum;
uint8_t length;

// Sampling Rate
uint8_t sampling_rate_array[3];
uint32_t sampling_rate;

// Messwertemaske
uint8_t measurement_mask_array[6];
uint32_t measurement_mask_low;
uint32_t measurement_mask_high;

uint8_t FIFObuffer[30];			// Puffer fr Empfangene Bytes
uint8_t FIFOcounter = 0;		// Zhlt wieviele Bytes empfangen wurden
uint8_t interruptcounter = 0;	// Zhlt von 0 rauf bis ringcounter

struct {						// Bitfeld fr Flags
	unsigned right_values:1;	// Sind die Werte richtig? Wenn 1, dann sind die Werte richtig
								// Wenn 0 war eine Prfsumme falsch ==> Keine Ausgabe bei Funktionen
} flag;



/*******************************************************
	Funktion :		Initialisieren von USART
	Eigenschaften:	8 Bit / 1 Stop / kein Parity
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void usart_init(void)
{
	// Baudrate einstellen
	UBRRH = (uint8_t) ((F_OSC / (BAUD_RATE * 16L) - 1)>>8);
    UBRRL = (uint8_t) (F_OSC / (BAUD_RATE * 16L) - 1);

	//Stelle Frameformat ein: 8 bit, 1 Stop, kein Parity
	UCSRC = (1 << URSEL) | (3 << UCSZ0) | (1 << UPM0) | (0 << USBS);

	//Doppelte Geschwindigkeit
	//UCSRA |= (1 << U2X);

	// Schalte Senden und Empfangen ein
	UCSRB = (1 << RXCIE) | (1 << TXEN) | (1 << RXEN);  // Aktiviere USART Interrupt

	//Stelle Seq_Nr und Pruefsumme auf 0;
	seq_nr = 0;
	checksum = 0;
}




/*******************************************************
	Funktion :		Errorfunktion von USART
	Eigenschaften:	Ruft allgemeine Error Routine auf
	bergabewert:	1 Byte Error Code
	Rckgabewert:	-
********************************************************/
void usart_error(uint8_t error_code)
{
	// Error Routine
	programm_error(error_code);
}



/*******************************************************
	Funktion :		Senden von Daten
	Eigenschaften:	Sendet 1 Byte
	bergabewert:	1 Byte Daten
	Rckgabewert:	-
********************************************************/
void usart_send(uint8_t data)
{
	// Warte bis bertragungs-Puffer leer ist (letzte bertragung fertig ist)
	while (!(UCSRA & (1<<UDRE)));

	//bergebene Daten in Data-Register geben (Daten bertragen)
	UDR = data;
}



/*******************************************************
	Funktion :		Senden von Daten
	Eigenschaften:	Sendet 3 Byte
	bergabewert:	3 Byte Daten
	Rckgabewert:	-
********************************************************/
void usart_send_24bit(uint32_t data)
{
	uint32_t temp;

	while (!(UCSRA & (1<<UDRE)));			// Warte bis bertragungs-Puffer leer ist	
	temp = data & 0xFF0000UL;				// Maskiere hohes Byte
	temp = temp >> 16;						// UDR ist nur 8 Bit gro => Schiebe wichtigen Bits auf 0-7 Position
	UDR = temp;								// Berechnete Daten in Data-Register geben (Daten bertragen)

	while (!(UCSRA & (1<<UDRE)));			// Warte bis bertragungs-Puffer leer ist	
	temp = data & 0xFF00UL;					// Maskiere mittleres Byte
	temp = temp >> 8;						// UDR ist nur 8 Bit gro => Schiebe wichtigen Bits auf 0-7 Position
	UDR = temp;								// Berechnete Daten in Data-Register geben (Daten bertragen)

	while (!(UCSRA & (1<<UDRE)));			// Warte bis bertragungs-Puffer leer ist	
	temp = data & 0xFFUL;					// Maskiere niedriges Byte
	UDR = temp;								// bergebene Daten in Data-Register geben (Daten bertragen)
}



/*******************************************************
	Funktion :		Empfangen von Daten
	Eigenschaften:	Empfangt 1 Byte
	bergabewert:	-
	Rckgabewert:	1 Byte Daten
********************************************************/
uint8_t usart_get(void)	
{
	//Warte bis Daten erhalten
	while ( !(UCSRA & ( 1<<RXC)) );

	//Hole Daten vom Puffer und gib sie zurck
	return UDR;
}




/*******************************************************
	Funktion :		Checkt ob neue Daten im Puffer
	Eigenschaften:	1 = ja			0 = nein
	bergabewert:	-
	Rckgabewert:	siehe Eigenschaften
********************************************************/
uint8_t usart_check(void)
{
	// Sind neue Daten im Buffer? 1 = ja ; 0 = nein
	uint8_t temp = (UCSRA & ( 1<<RXC));
	
	if(0b10000000 == temp)
	{
		return 1;			// Daten vorhanden
	}
	return 0;				// Keine Date vorhanden
}




/*******************************************************
	Funktion :		Seq Nr erniedrigen
	Eigenschaften:	0-255 (seq_nr)
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void usart_seq_minus(void)
{
	//Seq Nr erniedrigen oder auf 0 setzen
	if(seq_nr == 0)
	{
		seq_nr = 255;
	}
	else
	{
		seq_nr--;
	}
}



/*******************************************************
	Funktion :		Seq Nr zurcksetzen
	Eigenschaften:	seq_nr = 0
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void usart_seq_reset(void)
{
	seq_nr = 0;
}



/*******************************************************
	Funktion :		Seq Nr erhhen
	Eigenschaften:	0-255 (seq_nr)
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void usart_seq_plus(void)
{
	//Seq Nr erhhen oder auf 0 setzen
	if(seq_nr == 255)
	{
		seq_nr = 0;
	}
	else
	{
		seq_nr++;
	}
}



/*******************************************************
	Funktion :		Sendet seq_nr
	Eigenschaften:	Sendet 0-255 (seq_nr)
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void usart_send_seq_nr(void)
{
	// Sende Seq Nr.
	usart_send(seq_nr);

	//Seq Nr erhhen oder auf 0 setzen
	usart_seq_plus();
}




/*******************************************************
	Funktion :		Sende "Neue Datei"-Befehl (Nr. 0x10)
	Eigenschaften:	FF/SeqNr/0x05/0x10/Name/+
	bergabewert:	Name(Jahr,Monat,Tag,Stunde) = 4 Byte
	Rckgabewert:	-
********************************************************/
void usart_new_data(uint8_t year, uint8_t month, uint8_t date, uint8_t hours)
{
	length = 0x05;
	checksum = 0;

	// Sende FF (Start)
	usart_send(COMMAND_START);

	// Sende Seq Nr.
	usart_send_seq_nr();

	// Sende die Lnge 0x05 (also Lnge 5 Byte)
	usart_send(length);

	// Sende Befehlscode 0x10 (Neue Datei)
	usart_send(COMMAND_NEW_DATA);

	//Sende Name
	usart_send(year);
	usart_send(month);
	usart_send(date);
	usart_send(hours);

	//Berechne Pruefsumme
	checksum = length + COMMAND_NEW_DATA + year + month + date + hours;

	//Sende Pruefsumme
	usart_send(checksum);
}




/*******************************************************
	Funktion :		Sende "Neue Zeile"-Befehl (Nr. 0x11)
	Eigenschaften:	FF/SeqNr/Lnge(variabel)/0x11/Daten/+
	bergabewert:	Daten (variable Lnge)
	Rckgabewert:	-
********************************************************/
void usart_new_line(uint8_t measured_length,uint32_t measured[40], struct time rtc)								
{
	length = 0;
	checksum = 0;
	uint32_t temp;

	// Sende FF (Start)
	usart_send(COMMAND_START);

	// Sende Seq Nr.
	usart_send_seq_nr();

	// Berechne die variable Lnge 											
	length = 0x01 + 0x02 + measured_length*3;	//Befehlscode + Zeit(Sekunden,Minuten) + Messdaten * 3 Byte
 
	// Sende die Lnge
	usart_send(length);

	// Sende Befehlscode 0x11 (Neue Zeile)
	usart_send(COMMAND_NEW_LINE);

	//Sende Daten	
	usart_send(rtc.minutes);					// bertrage Minuten in BCD
	checksum += rtc.minutes;					// Addiere zu Checksumme

	usart_send(rtc.seconds);					// bertrage Sekunden in BCD
	checksum += rtc.seconds;					// Addiere zu Checksumme

	for(uint8_t elementcounter = 0; elementcounter < measured_length; ++elementcounter) // Durchlaufe alle gemessenen Werte
	{
		usart_send_24bit(measured[elementcounter]);
		
		temp = measured[elementcounter] & 0xFF0000UL;			// Maskiere hchstes Byte
		temp = temp >> 16;										// Auf Position 0-7 schieben
		checksum += temp;										// Addiere zu Checksumme

		temp = measured[elementcounter] & 0xFF00UL;				// Maskiere mittleres Byte
		temp = temp >> 8;										// Auf Position 0-7 schieben
		checksum += temp;										// Addiere zu Checksumme

		temp = measured[elementcounter] & 0xFFUL;				// Maskiere niedrigstes Byte
		checksum += temp;										// Addiere zu Checksumme
	}

	//Berechne restliche Pruefsumme
	checksum += COMMAND_NEW_LINE + length;

	//Sende Pruefsumme
	usart_send(checksum);
}




/*******************************************************
	Funktion :		Sende "Datei fertig"-Befehl (Nr. 0x12)
	Eigenschaften:	FF/SeqNr/0x05/0x12/Name/+
	bergabewert:	Name(Jahr,Monat,Tag,Stunde) = 4 Byte
	Rckgabewert:	-
********************************************************/
void usart_data_finish(uint8_t jahr, uint8_t monat, uint8_t tag, uint8_t stunde)
{
	length = 0x05;
	checksum = 0;

	// Sende FF (Start)
	usart_send(COMMAND_START);

	// Sende Seq Nr.
	usart_send_seq_nr();

	// Sende die Lnge 0x05 (also Lnge 5 Byte)
	usart_send(length);

	// Sende Befehlscode 0x12 (Datei Fertig)
	usart_send(COMMAND_DATA_FINISH);

	//Sende Name
	usart_send(jahr);
	usart_send(monat);
	usart_send(tag);
	usart_send(stunde);

	//Berechne Pruefsumme
	checksum = length + COMMAND_DATA_FINISH + jahr + monat + tag + stunde;

	//Sende Pruefsumme
	usart_send(checksum);
}




/*******************************************************
	Funktion :		Sende "keine Dateien mehr"-Befehl (Nr. 0x13)
	Eigenschaften:	FF/SeqNr/0x01/0x13/+
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void usart_no_more_data(void)
{
	length = 0x01;
	checksum = 0;

	// Sende FF (Start)
	usart_send(COMMAND_START);

	// Sende Seq Nr.
	usart_send_seq_nr();

	// Sende die Lnge 0x01 (also Lnge 1 Byte)
	usart_send(length);

	// Sende Befehlscode 0x13 (keine Dateien mehr)
	usart_send(COMMAND_NO_MORE_DATA);

	//Berechne Pruefsumme
	checksum = length + COMMAND_NO_MORE_DATA;

	//Sende Pruefsumme
	usart_send(checksum);
}




/*******************************************************
	Funktion :		Sende "Send ACK"-Befehl (Nr. 0x14)
	Eigenschaften:	FF/SeqNr/0x02/0x14/alte Seq/+
	bergabewert:	-
	Rckgabewert:	-
********************************************************/
void usart_send_ack(void)
{
	length = 0x02;
	checksum = 0;

	// Sende FF (Start)
	usart_send(COMMAND_START);

	// Sende Seq Nr.
	usart_send_seq_nr();

	// Sende die Lnge
	usart_send(length);

	// Sende Befehlscode 0x14 (Sende ACK)
	usart_send(COMMAND_SEND_ACK);

	// Sende alte Sequenznr.
	usart_send((seq_nr-2));					// -2, da +1 in diesem Packet war und +1 in erhaltenen Packet war

	//Berechne Pruefsumme
	checksum = length + COMMAND_SEND_ACK + (seq_nr-2);

	//Sende Pruefsumme
	usart_send(checksum);
}




/*******************************************************
	Funktion :		Sende "Status Antwort"-Befehl (Nr. 0x15)
	Eigenschaften:	FF/SeqNr/0x02/0x15/ErrorCode/+
	bergabewert:	1 Byte ErrorCode
	Rckgabewert:	-
********************************************************/
void usart_status_response(uint8_t error_code)
{
	length = 0x02;
	checksum = 0;

	// Sende FF (Start)
	usart_send(COMMAND_START);

	// Sende Seq Nr.
	usart_send_seq_nr();

	// Sende die Lnge
	usart_send(length);

	// Sende Befehlscode 0x15 (Sende Status Response)
	usart_send(COMMAND_STATUS_RESPONSE);

	// Sende den Error Code
	usart_send(error_code);

	//Berechne Pruefsumme
	checksum = length + COMMAND_STATUS_RESPONSE + error_code;

	//Sende Pruefsumme
	usart_send(checksum);
}



/*******************************************************
	Funktion :		Hole Packet / Reagiere auf Befehlscode
	Eigenschaften:	Gib Befehlsnummer zurck
	bergabewert:	-
	Rckgabewert:	1 Byte Befehlsnummer (0xFF = Fehler)
********************************************************/
uint8_t usart_receive_packet(void)
{	
	uint8_t get_start;
	uint8_t get_seq;
	uint8_t get_command;
	uint8_t data;
	length = 0;

	get_start = usart_interrupt_get();						// Hole Start-Signal
	get_seq = usart_interrupt_get();
	length = usart_interrupt_get();
	get_command = usart_interrupt_get();

	if(get_start == COMMAND_START) 							// Wurde Start bermittelt?
	{
		if(get_seq == seq_nr)								// Stimmen Seq Nummern berein?
		{
			switch(get_command)
			{
				case COMMAND_SEND_ALL_DATA:					//Schicke alle Daten
				{
					data = usart_interrupt_get();				// Hole Pruefsumme
					checksum = length + COMMAND_SEND_ALL_DATA;	// Berechne Pruefsumme

					if (data == checksum)
					{
						usart_seq_plus();					// Erhhe Sequenz nr.
						return COMMAND_SEND_ALL_DATA;		// Return Befehlsnr.
					}
					else
					{
						usart_error(EC_USART_SEND_ALL_DATA);
						return 0xFF;						// Fehler
					}
				} break;

				case COMMAND_LIVE_MODUS:					//Live bertragung
				{
					data = usart_interrupt_get();			// Hole Pruefsumme
					checksum = length + COMMAND_LIVE_MODUS;	// Berechne Pruefsumme
	
					if (data == checksum)
					{
						usart_seq_plus();					// Erhhe Sequenz nr.
						return COMMAND_LIVE_MODUS;			// Return Befehlsnr.
					}
					else
					{
						usart_error(EC_USART_LIVE_MODUS);
						return 0xFF;						// Fehler
					}
				} break;

				case COMMAND_LIVE_MODUS_STOP:				// Live Stopp
				{
					data = usart_interrupt_get();					// Hole Pruefsumme
					checksum = length + COMMAND_LIVE_MODUS_STOP;	// Berechne Pruefsumme

					if (data == checksum)
					{
						usart_seq_plus();					// Erhhe Sequenz nr.
						return COMMAND_LIVE_MODUS_STOP;		// Return Befehlsnr.
					}
					else
					{
						usart_error(EC_USART_LIVE_MODUS_STOP);
						return 0xFF;						// Fehler
					}
				} break;

				case COMMAND_RECEIVE_TIME: 					//Uhrzeit empfangen
				{
					//Hole Uhrzeit-Daten
					pc.year = usart_interrupt_get();
					pc.month = usart_interrupt_get();
					pc.date = usart_interrupt_get();
					pc.hours = usart_interrupt_get();
					pc.minutes = usart_interrupt_get();
					pc.seconds = usart_interrupt_get();

					data = usart_interrupt_get();			// Hole Pruefsumme
					checksum = length + COMMAND_RECEIVE_TIME + pc.year + pc.month + pc.date + pc.hours 
								+ pc.minutes + pc.seconds; 	// Berechne Pruefsumme

					if (data == checksum)
					{
						usart_seq_plus();					// Erhhe Sequenz nr.
						flag.right_values = 1;				// Werte waren richtig (wegen Prfsumme)
						return COMMAND_RECEIVE_TIME;		// Return Befehlsnr.
					}
					else
					{
						flag.right_values = 0;				// Werte waren falsch (wegen Prfsumme)
						usart_error(EC_USART_RECEIVE_TIME);
						return 0xFF;						// Fehler
					}
				} break;

				case COMMAND_RECEIVE_ACK: 					// ACK empfangen
				{
					pc_seq_nr = usart_interrupt_get();		// Hole Seq Nr
					data = usart_interrupt_get();			// Hole Pruefsumme
					checksum = length + COMMAND_RECEIVE_ACK + pc_seq_nr; // Berechne Pruefsumme
					
					if(pc_seq_nr == (seq_nr-1))
					{
						if(data == checksum)
						{
							usart_seq_plus();					// Erhhe Sequenz nr.
							return COMMAND_RECEIVE_ACK;			// Return Befehlsnummer
						}
						else
						{
							usart_error(EC_USART_RECEIVE_ACK);
							return 0xFF;						// Fehler
						}
					}
					else
					{
						usart_error(EC_USART_WRONG_OLD_SEQ);
						return 0xFF;							// Fehler
					}						
				} break;

				case COMMAND_PARAMETERS:					//Paramters
				{
					// Hole Startzeit
					start.year = usart_interrupt_get();
					start.month = usart_interrupt_get();
					start.date = usart_interrupt_get();
					start.hours = usart_interrupt_get();
					start.minutes = usart_interrupt_get();
					start.seconds = usart_interrupt_get();

					//Hole Endzeit
					end.year = usart_interrupt_get();
					end.month = usart_interrupt_get();
					end.date = usart_interrupt_get();
					end.hours = usart_interrupt_get();
					end.minutes = usart_interrupt_get();
					end.seconds = usart_interrupt_get();

					// Hole Sampling Rate
					// Speichere in Zwischenvariablen fr Pruefsumme
					sampling_rate_array[0] = usart_interrupt_get();
					sampling_rate_array[1] = usart_interrupt_get();
					sampling_rate_array[2] = usart_interrupt_get();

					sampling_rate = sampling_rate_array[0];				// Schreibe 1tes Byte
					sampling_rate = (sampling_rate << 8);				// Verschiebe 1tes Byte
					sampling_rate |= sampling_rate_array[1];			// Schreibe 2tes Byte
					sampling_rate = (sampling_rate << 8);				// Verschiebe 2tes Byte
					sampling_rate |= sampling_rate_array[2];			// Schreibe 3tes Byte
					
					// Hole Messwertemaske
					// Speichere in Zwischenvariablen fr Pruefsumme
					measurement_mask_array[0] = usart_interrupt_get();
					measurement_mask_array[1] = usart_interrupt_get();
					measurement_mask_array[2] = usart_interrupt_get();
					measurement_mask_array[3] = usart_interrupt_get();
					measurement_mask_array[4] = usart_interrupt_get();
					measurement_mask_array[5] = usart_interrupt_get();

					measurement_mask_high = measurement_mask_array[0];	// Wie bei sampling rate (oben)
					measurement_mask_high = (measurement_mask_high << 8);
					measurement_mask_high |= measurement_mask_array[1];

					measurement_mask_low = measurement_mask_array[2];
					measurement_mask_low = (measurement_mask_low << 8);
					measurement_mask_low |= measurement_mask_array[3];
					measurement_mask_low = (measurement_mask_low << 8);
					measurement_mask_low |= measurement_mask_array[4];
					measurement_mask_low = (measurement_mask_low << 8);
					measurement_mask_low |= measurement_mask_array[5];
					
					data = usart_interrupt_get();									// Hole Pruefsumme
					
					// Berechne die Pruefsumme
					checksum = length + COMMAND_PARAMETERS + start.year + start.month + start.date + start.hours + start.minutes + start.seconds
							   + end.year + end.month + end.date + end.hours + end.minutes + end.seconds + sampling_rate_array[0]
							   + sampling_rate_array[1] + sampling_rate_array[2] + measurement_mask_array[0] 
							   + measurement_mask_array[1] + measurement_mask_array[2] + measurement_mask_array[3] 
							   + measurement_mask_array[4] + measurement_mask_array[5];

					if(data == checksum)
					{
						usart_seq_plus();								// Erhhe Sequenz nr.
						flag.right_values = 1;							// Werte richtig (wegen Prfsumme)
						return COMMAND_PARAMETERS;						// Return Befehlsnr.
					}
					else
					{
						flag.right_values = 0;							// Werte falsch (wegen Prfsumme)
						usart_error(EC_USART_COMMAND_PARAMETERS);
						return 0xFF;									// Fehler
					}
				} break;
				
				case COMMAND_STATUS_REQUEST:					// Status-senden Aufforderung
				{
					data = usart_interrupt_get();				// Hole Pruefsumme
					checksum = length + COMMAND_STATUS_REQUEST;	// Berechne Pruefsumme

					if (data == checksum)
					{
						usart_seq_plus();					// Erhhe Sequenz nr.
						return COMMAND_STATUS_REQUEST;		// Return Befehlsnr.
					}
					else
					{
						usart_error(EC_USART_STATUS_REQUEST);
						return 0xFF;						// Fehler
					}
				} break;
				
				case COMMAND_RESET:
				{
					goto packet_reset;
				}

				case COMMAND_CALIBRATE:
				{
					data = usart_interrupt_get();			// Hole Pruefsumme
					checksum = length + COMMAND_CALIBRATE;	// Berechne Pruefsumme

					if (data == checksum)
					{
						usart_seq_plus();					// Erhhe Sequenz nr.
						return COMMAND_CALIBRATE;			// Return Befehlsnr.
					}
					else
					{
						usart_error(EC_USART_STATUS_REQUEST);
						return 0xFF;						// Fehler
					}
				}

				default:						// Falscher Befehlscode
				{
					usart_error(EC_USART_WRONG_CODE);
					return 0xFF;				// Fehler
				}
			}
		}
		else
		{
			if(get_command == COMMAND_RESET)
			{
				packet_reset:
				data = usart_interrupt_get();			// Hole Pruefsumme
				checksum = length + COMMAND_RESET;		// Berechne Pruefsumme
				
				if (data == checksum)
				{
					usart_seq_plus();					// Erhhe Sequenz nr.
					return COMMAND_RESET;				// Return Befehlsnr.
				}
				else
				{
					usart_error(EC_USART_RESET);
					return 0xFF;						// Fehler
				}

			}
			else
			{
				usart_error(EC_USART_WRONG_SEQ);	// Falsche Seq Nr. ==> Error
				return 0xFF;						// FEHLER
			}
		}
	}
	else
	{
		usart_error(EC_USART_NO_START);			// Kein Start erhalten ==> Error
		return 0xFF;							// FEHLER
	}
}




/*******************************************************
	Funktion :		Liefert Uhrzeit zurck
	Eigenschaften:	Jahr/Monat/Tag/Stunde/Minute/Sekunde
	bergabewert:	"&" Uhrzeit damit als Rckgabewert!
	Rckgabewert:	1 Byte ob Werte richtig (1 = ja)
********************************************************/
uint8_t usart_receive_time(struct time *t)
{
	if(flag.right_values == 1)
	{
		struct time temp;
		temp.seconds = pc.seconds;
		temp.minutes = pc.minutes;
		temp.hours = pc.hours;
		temp.date = pc.date;
		temp.month = pc.month;
		temp.year = pc.year;
		*t = temp;
		return 1;				// Werte stimmen
	}
	else
	{
		return 0;				// Es gab einen Fehler bei Prfsumme
	}
}




/*******************************************************
	Funktion :	Stimmen Seq Nr. berein? (bei ACK)	
	Eigenschaften: 1 = Ja, 0 = Nein
	bergabewert: -	
	Rckgabewert: 1 Byte (1=Ja,0=Nein)	
********************************************************/
uint8_t usart_seq_nr_equals(void)
{
	if(flag.right_values == 1)
	{
		if(pc_seq_nr == seq_nr)
		{
			return 1;			// Seq_nr stimmen berein
		}
		else
		{
			return 0;			// Seq_nr stimmen nicht berein
		}
	}
	else
	{
		return 0;				// Es gab Fehler bei Prfsumme
	}
}




/*******************************************************
	Funktion :	Liefert "Parameter" zurck
	Eigenschaften: Startzeit,Endzeit,SamplingRate,Messmaske
	bergabewert: "Paramter"
	Rckgabewert: 1 Byte ob Werte richtig (1 = ja)
********************************************************/
uint8_t usart_parameters(struct time *start2, struct time *end2, uint32_t *sampling_rate2, uint32_t *measurement_mask1,uint32_t *measurement_mask2)
{
	if(flag.right_values == 1)
	{
		*start2 = start;

		*end2 = end;

		*sampling_rate2 	= sampling_rate,
		
		*measurement_mask1 	= measurement_mask_low;
		*measurement_mask2 	= measurement_mask_high;

		return 1;				// Werte stimmen
	}
	else
	{
		return 0;				// Es gab Fehler bei Prfsumme
	}
}


/*******************************************************
	Funktion :	Interrupt-Routine
	Eigenschaften: Speichert USART Wert in einem Buffer
	bergabewert: Vektor auf Interrupt
	Rckgabewert: -
********************************************************/
ISR(USART_RXC_vect)
{
	FIFObuffer[FIFOcounter] = usart_get();
	FIFOcounter++;
}




/*******************************************************
	Funktion :	Liefert nchste FIFObuffer Wert
	Eigenschaften: -
	bergabewert: -
	Rckgabewert: 1 Byte gepufferter USART-Daten
********************************************************/
uint8_t usart_interrupt_get(void)
{
	uint8_t data;

	data = FIFObuffer[interruptcounter];		// Hole Daten aus FIFOpuffer
	FIFObuffer[interruptcounter] = 0;			// Setze FIFOpuffer zurck

	_delay_ms(50);

	if(interruptcounter < (FIFOcounter-1))		// Gibt es noch Daten im FIFOpuffer? (-1 weil immer auf nchstes zeigt)
	{
		interruptcounter++;						// Auf nchstes Byte im Buffer zeigen
	}
	else
	{
		usart_reset_counters();
	}
	return data;								// Gib Daten zurck
}




/*******************************************************
	Funktion :	Setzt die Ringbuffer Zhler zurck
	Eigenschaften: -
	bergabewert: -
	Rckgabewert: -
********************************************************/
void usart_reset_counters(void)
{
	interruptcounter = 0;					// Auf Anfangszustand zurcksetzen
	FIFOcounter = 0;						// Auf Anfangszustand zurcksetzen
}



/*******************************************************
	Funktion :	Checkt ob noch Daten vorhanden sind
	Eigenschaften: -
	bergabewert: -
	Rckgabewert: 1 Byte ob Daten vorhanden (1 = ja)
********************************************************/
uint8_t usart_interrupt_check(void)
{
	if(FIFOcounter > 0)							// Gibt es Daten im FIFOpuffer?
		return 1;
	else
		return 0;
}
